package com.sun.corba.se.impl.util;

import java.util.Enumeration;
import java.util.Dictionary;

public final class IdentityHashtable extends Dictionary
{
    private transient IdentityHashtableEntry[] table;
    private transient int count;
    private int threshold;
    private float loadFactor;
    
    public IdentityHashtable(int n, float loadFactor) {
        if (n <= 0 || (double)loadFactor <= 0.0) {
            throw new IllegalArgumentException();
        }
        this.loadFactor = loadFactor;
        this.table = new IdentityHashtableEntry[n];
        this.threshold = (int)((float)n * loadFactor);
        return;
    }
    
    public IdentityHashtable(int n) {
        this(n, 0.75f);
    }
    
    public IdentityHashtable() {
        this(101, 0.75f);
    }
    
    @Override
    public int size() {
        return this.count;
    }
    
    @Override
    public boolean isEmpty() {
        return this.count == 0;
    }
    
    @Override
    public Enumeration keys() {
        return new IdentityHashtableEnumerator(this.table, true);
    }
    
    @Override
    public Enumeration elements() {
        return new IdentityHashtableEnumerator(this.table, false);
    }
    
    public boolean contains(Object o) {
        if (o == null) {
            throw new NullPointerException();
        }
        IdentityHashtableEntry[] table = this.table;
        int length = table.length;
        while (length-- > 0) {
            IdentityHashtableEntry next = table[length];
            while (next != null) {
                if (next.value == o) {
                    return true;
                }
                next = next.next;
            }
        }
        return false;
    }
    
    public boolean containsKey(Object o) {
        IdentityHashtableEntry[] table = this.table;
        int identityHashCode = System.identityHashCode(o);
        IdentityHashtableEntry next = table[(identityHashCode & Integer.MAX_VALUE) % table.length];
        while (next != null) {
            if (next.hash == identityHashCode && next.key == o) {
                return true;
            }
            next = next.next;
        }
        return false;
    }
    
    @Override
    public Object get(Object o) {
        IdentityHashtableEntry[] table = this.table;
        int identityHashCode = System.identityHashCode(o);
        IdentityHashtableEntry next = table[(identityHashCode & Integer.MAX_VALUE) % table.length];
        while (next != null) {
            if (next.hash == identityHashCode && next.key == o) {
                return next.value;
            }
            next = next.next;
        }
        return null;
    }
    
    protected void rehash() {
        int length = this.table.length;
        IdentityHashtableEntry[] table = this.table;
        int n = length * 2 + 1;
        IdentityHashtableEntry[] table2 = new IdentityHashtableEntry[n];
        this.threshold = (int)((float)n * this.loadFactor);
        this.table = table2;
        while (length-- > 0) {
            int n2;
            for (IdentityHashtableEntry next = table[length]; next != null; , next = next.next, n2 = (next.hash & Integer.MAX_VALUE) % n, next.next = table2[n2], table2[n2] = next) {}
        }
    }
    
    @Override
    public Object put(Object key, Object o) {
        if (o == null) {
            throw new NullPointerException();
        }
        IdentityHashtableEntry[] table = this.table;
        int identityHashCode = System.identityHashCode(key);
        int n = (identityHashCode & Integer.MAX_VALUE) % table.length;
        IdentityHashtableEntry next = table[n];
        while (next != null) {
            if (next.hash == identityHashCode && next.key == key) {
                Object value = next.value;
                next.value = o;
                return value;
            }
            next = next.next;
        }
        if (this.count >= this.threshold) {
            this.rehash();
            return this.put(key, o);
        }
        IdentityHashtableEntry o2 = new IdentityHashtableEntry();
        o2.hash = identityHashCode;
        o2.key = key;
        o2.value = o;
        o2.next = table[n];
        table[n] = o2;
        ++this.count;
        return null;
    }
    
    @Override
    public Object remove(Object o) {
        IdentityHashtableEntry[] table = this.table;
        int identityHashCode = System.identityHashCode(o);
        int n = (identityHashCode & Integer.MAX_VALUE) % table.length;
        IdentityHashtableEntry next = table[n];
        IdentityHashtableEntry identityHashtableEntry = null;
        while (next != null) {
            if (next.hash == identityHashCode && next.key == o) {
                if (identityHashtableEntry != null) {
                    identityHashtableEntry.next = next.next;
                }
                else {
                    table[n] = next.next;
                }
                --this.count;
                return next.value;
            }
            else {
                identityHashtableEntry = next;
                next = next.next;
            }
        }
        return null;
    }
    
    public void clear() {
        IdentityHashtableEntry[] table = this.table;
        int length = table.length;
        while (--length >= 0) {
            table[length] = null;
        }
        this.count = 0;
    }
    
    @Override
    public String toString() {
        int n = this.size() - 1;
        StringBuffer sb = new StringBuffer();
        Enumeration<Object> keys = this.keys();
        Enumeration<Object> elements = this.elements();
        sb.append("{");
        for (int i = 0; i <= n; ++i) {
            sb.append(keys.nextElement().toString() + "=" + elements.nextElement().toString());
            if (i < n) {
                sb.append(", ");
            }
        }
        sb.append("}");
        return sb.toString();
    }
}
